# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import List, Optional
from typing_extensions import Annotated
from instana_client.models.application_result import ApplicationResult
from instana_client.models.endpoint import Endpoint
from instana_client.models.endpoint_result import EndpointResult
from instana_client.models.service import Service
from instana_client.models.service_result import ServiceResult

from instana_client.api_client import ApiClient, RequestSerialized
from instana_client.api_response import ApiResponse
from instana_client.rest import RESTResponseType


class ApplicationResourcesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_application_endpoints(
        self,
        app_id: Annotated[Optional[StrictStr], Field(description="Application ID")] = None,
        service_id: Annotated[Optional[StrictStr], Field(description="Service ID")] = None,
        endpoint_id: Annotated[Optional[StrictStr], Field(description="Endpoint ID")] = None,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Type of Endpoint")] = None,
        technologies: Annotated[Optional[List[StrictStr]], Field(description="List of technologies")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[EndpointResult, Endpoint]:
        """Get endpoints

        Use this API endpoint if one wants to retrieve a list of Endpoints. A use case could be to view the endpoint id of an Endpoint.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param app_id: Application ID (optional, used to filter endpoints by application)
        :type app_id: str
        :param service_id: Service ID (optional, used to filter endpoints by service)
        :type service_id: str
        :param endpoint_id: Endpoint ID (optional, used to get a specific endpoint)
        :type endpoint_id: str
        :param name_filter: Name of service
        :type name_filter: str
        :param types: Type of Endpoint
        :type types: List[str]
        :param technologies: List of technologies
        :type technologies: List[str]
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_endpoints_serialize(
            app_id=app_id,
            service_id=service_id,
            endpoint_id=endpoint_id,
            name_filter=name_filter,
            types=types,
            technologies=technologies,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        # Determine response type based on parameters
        if app_id and service_id and endpoint_id:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "Endpoint",
            }
        else:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "EndpointResult",
            }
        
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_application_endpoints_with_http_info(
        self,
        app_id: Annotated[Optional[StrictStr], Field(description="Application ID")] = None,
        service_id: Annotated[Optional[StrictStr], Field(description="Service ID")] = None,
        endpoint_id: Annotated[Optional[StrictStr], Field(description="Endpoint ID")] = None,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Type of Endpoint")] = None,
        technologies: Annotated[Optional[List[StrictStr]], Field(description="List of technologies")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[EndpointResult], ApiResponse[Endpoint]]:
        """Get endpoints

        Use this API endpoint if one wants to retrieve a list of Endpoints. A use case could be to view the endpoint id of an Endpoint.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param app_id: Application ID (optional, used to filter endpoints by application)
        :type app_id: str
        :param service_id: Service ID (optional, used to filter endpoints by service)
        :type service_id: str
        :param endpoint_id: Endpoint ID (optional, used to get a specific endpoint)
        :type endpoint_id: str
        :param name_filter: Name of service
        :type name_filter: str
        :param types: Type of Endpoint
        :type types: List[str]
        :param technologies: List of technologies
        :type technologies: List[str]
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_endpoints_serialize(
            app_id=app_id,
            service_id=service_id,
            endpoint_id=endpoint_id,
            name_filter=name_filter,
            types=types,
            technologies=technologies,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        # Determine response type based on parameters
        if app_id and service_id and endpoint_id:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "Endpoint",
            }
        else:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "EndpointResult",
            }
        
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_application_endpoints_without_preload_content(
        self,
        app_id: Annotated[Optional[StrictStr], Field(description="Application ID")] = None,
        service_id: Annotated[Optional[StrictStr], Field(description="Service ID")] = None,
        endpoint_id: Annotated[Optional[StrictStr], Field(description="Endpoint ID")] = None,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Type of Endpoint")] = None,
        technologies: Annotated[Optional[List[StrictStr]], Field(description="List of technologies")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get endpoints

        Use this API endpoint if one wants to retrieve a list of Endpoints. A use case could be to view the endpoint id of an Endpoint.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param app_id: Application ID (optional, used to filter endpoints by application)
        :type app_id: str
        :param service_id: Service ID (optional, used to filter endpoints by service)
        :type service_id: str
        :param endpoint_id: Endpoint ID (optional, used to get a specific endpoint)
        :type endpoint_id: str
        :param name_filter: Name of service
        :type name_filter: str
        :param types: Type of Endpoint
        :type types: List[str]
        :param technologies: List of technologies
        :type technologies: List[str]
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_endpoints_serialize(
            app_id=app_id,
            service_id=service_id,
            endpoint_id=endpoint_id,
            name_filter=name_filter,
            types=types,
            technologies=technologies,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        # Determine response type based on parameters
        if app_id and service_id and endpoint_id:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "Endpoint",
            }
        else:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "EndpointResult",
            }
        
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_application_endpoints_serialize(
        self,
        app_id,
        service_id,
        endpoint_id,
        name_filter,
        types,
        technologies,
        window_size,
        to,
        page,
        page_size,
        application_boundary_scope,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'types': 'multi',
            'technologies': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if name_filter is not None:
            
            _query_params.append(('nameFilter', name_filter))
            
        if types is not None:
            
            _query_params.append(('types', types))
            
        if technologies is not None:
            
            _query_params.append(('technologies', technologies))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('pageSize', page_size))
            
        if application_boundary_scope is not None:
            
            _query_params.append(('applicationBoundaryScope', application_boundary_scope))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # Build dynamic resource path based on app_id, service_id, and endpoint_id
        if app_id and service_id and endpoint_id:
            resource_path = f'/api/application-monitoring/applications;id={app_id}/services;id={service_id}/endpoints;id={endpoint_id}'
        elif app_id and service_id:
            resource_path = f'/api/application-monitoring/applications;id={app_id}/services;id={service_id}/endpoints'
        elif app_id:
            resource_path = f'/api/application-monitoring/applications;id={app_id}/services/endpoints'
        else:
            resource_path = '/api/application-monitoring/applications/services/endpoints'

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path=resource_path,
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_application_services(
        self,
        app_id: Annotated[Optional[StrictStr], Field(description="Application ID")] = None,
        service_id: Annotated[Optional[StrictStr], Field(description="Service ID")] = None,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service (partial match allowed)")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        include_snapshot_ids: Annotated[Optional[StrictBool], Field(description="Include snapshot ids in the results")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ServiceResult, Service]:
        """Get applications/services

        Use this API endpoint if one wants to retrieve a list of services in an Application Perspective. A use case could be to retrieve all service ids present in an Application Perspective.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param app_id: Application ID (optional, used to filter services by application)
        :type app_id: str
        :param service_id: Service ID (optional, used to filter specific service)
        :type service_id: str
        :param name_filter: Name of service (partial match allowed)
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param include_snapshot_ids: Include snapshot ids in the results
        :type include_snapshot_ids: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_services_serialize(
            app_id=app_id,
            service_id=service_id,
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            include_snapshot_ids=include_snapshot_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        # Determine response type based on parameters
        if app_id and service_id:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "Service",
            }
        else:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "ServiceResult",
            }
        
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_application_services_with_http_info(
        self,
        app_id: Annotated[Optional[StrictStr], Field(description="Application ID")] = None,
        service_id: Annotated[Optional[StrictStr], Field(description="Service ID")] = None,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service (partial match allowed)")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        include_snapshot_ids: Annotated[Optional[StrictBool], Field(description="Include snapshot ids in the results")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[ServiceResult], ApiResponse[Service]]:
        """Get applications/services

        Use this API endpoint if one wants to retrieve a list of services in an Application Perspective. A use case could be to retrieve all service ids present in an Application Perspective.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param app_id: Application ID (optional, used to filter services by application)
        :type app_id: str
        :param service_id: Service ID (optional, used to filter specific service)
        :type service_id: str
        :param name_filter: Name of service (partial match allowed)
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param include_snapshot_ids: Include snapshot ids in the results
        :type include_snapshot_ids: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_services_serialize(
            app_id=app_id,
            service_id=service_id,
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            include_snapshot_ids=include_snapshot_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        # Determine response type based on parameters
        if app_id and service_id:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "Service",
            }
        else:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "ServiceResult",
            }
        
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_application_services_without_preload_content(
        self,
        app_id: Annotated[Optional[StrictStr], Field(description="Application ID")] = None,
        service_id: Annotated[Optional[StrictStr], Field(description="Service ID")] = None,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service (partial match allowed)")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        include_snapshot_ids: Annotated[Optional[StrictBool], Field(description="Include snapshot ids in the results")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get applications/services

        Use this API endpoint if one wants to retrieve a list of services in an Application Perspective. A use case could be to retrieve all service ids present in an Application Perspective.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param app_id: Application ID (optional, used to filter services by application)
        :type app_id: str
        :param service_id: Service ID (optional, used to filter specific service)
        :type service_id: str
        :param name_filter: Name of service (partial match allowed)
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param include_snapshot_ids: Include snapshot ids in the results
        :type include_snapshot_ids: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_application_services_serialize(
            app_id=app_id,
            service_id=service_id,
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            include_snapshot_ids=include_snapshot_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        # Determine response type based on parameters
        if app_id and service_id:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "Service",
            }
        else:
            _response_types_map: Dict[str, Optional[str]] = {
                '200': "ServiceResult",
            }
        
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_application_services_serialize(
        self,
        app_id,
        service_id,
        name_filter,
        window_size,
        to,
        page,
        page_size,
        application_boundary_scope,
        include_snapshot_ids,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if name_filter is not None:
            
            _query_params.append(('nameFilter', name_filter))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('pageSize', page_size))
            
        if application_boundary_scope is not None:
            
            _query_params.append(('applicationBoundaryScope', application_boundary_scope))
            
        if include_snapshot_ids is not None:
            
            _query_params.append(('includeSnapshotIds', include_snapshot_ids))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # Build dynamic resource path based on app_id and service_id
        if app_id and service_id:
            resource_path = f'/api/application-monitoring/applications;id={app_id}/services;id={service_id}'
        elif app_id:
            resource_path = f'/api/application-monitoring/applications;id={app_id}/services'
        else:
            resource_path = '/api/application-monitoring/applications/services'

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path=resource_path,
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_applications(
        self,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of application")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApplicationResult:
        """Get applications

        Use this API endpoint if one wants to retrieve a list of Application Perspectives. A use case could be to view the application id of an Application Perspective.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param name_filter: Name of application
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_applications_serialize(
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ApplicationResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_applications_with_http_info(
        self,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of application")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ApplicationResult]:
        """Get applications

        Use this API endpoint if one wants to retrieve a list of Application Perspectives. A use case could be to view the application id of an Application Perspective.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param name_filter: Name of application
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_applications_serialize(
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ApplicationResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_applications_without_preload_content(
        self,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of application")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        application_boundary_scope: Annotated[Optional[StrictStr], Field(description="Filter for application scope, i.e: INBOUND or ALL")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get applications

        Use this API endpoint if one wants to retrieve a list of Application Perspectives. A use case could be to view the application id of an Application Perspective.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param name_filter: Name of application
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param application_boundary_scope: Filter for application scope, i.e: INBOUND or ALL
        :type application_boundary_scope: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_applications_serialize(
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            application_boundary_scope=application_boundary_scope,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ApplicationResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_applications_serialize(
        self,
        name_filter,
        window_size,
        to,
        page,
        page_size,
        application_boundary_scope,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if name_filter is not None:
            
            _query_params.append(('nameFilter', name_filter))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('pageSize', page_size))
            
        if application_boundary_scope is not None:
            
            _query_params.append(('applicationBoundaryScope', application_boundary_scope))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/application-monitoring/applications',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_services(
        self,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        include_snapshot_ids: Annotated[Optional[StrictBool], Field(description="Include snapshot ids in the results")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ServiceResult:
        """Get services

        Use this API endpoint if one wants to retrieve a list of Services. A use case could be to view the service id of a Service.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param name_filter: Name of service
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param include_snapshot_ids: Include snapshot ids in the results
        :type include_snapshot_ids: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_services_serialize(
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            include_snapshot_ids=include_snapshot_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ServiceResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_services_with_http_info(
        self,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        include_snapshot_ids: Annotated[Optional[StrictBool], Field(description="Include snapshot ids in the results")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ServiceResult]:
        """Get services

        Use this API endpoint if one wants to retrieve a list of Services. A use case could be to view the service id of a Service.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param name_filter: Name of service
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param include_snapshot_ids: Include snapshot ids in the results
        :type include_snapshot_ids: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_services_serialize(
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            include_snapshot_ids=include_snapshot_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ServiceResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_services_without_preload_content(
        self,
        name_filter: Annotated[Optional[StrictStr], Field(description="Name of service")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Size of time window in milliseconds")] = None,
        to: Annotated[Optional[StrictInt], Field(description="Timestamp since Unix Epoch in milliseconds of the end of the time window")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page number from results")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of items per page")] = None,
        include_snapshot_ids: Annotated[Optional[StrictBool], Field(description="Include snapshot ids in the results")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get services

        Use this API endpoint if one wants to retrieve a list of Services. A use case could be to view the service id of a Service.  For more information on Application Resources please access the https://developer.ibm.com/apis/catalog/instana--instana-rest-api/Applications#application-resources.

        :param name_filter: Name of service
        :type name_filter: str
        :param window_size: Size of time window in milliseconds
        :type window_size: int
        :param to: Timestamp since Unix Epoch in milliseconds of the end of the time window
        :type to: int
        :param page: Page number from results
        :type page: int
        :param page_size: Number of items per page
        :type page_size: int
        :param include_snapshot_ids: Include snapshot ids in the results
        :type include_snapshot_ids: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_services_serialize(
            name_filter=name_filter,
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            include_snapshot_ids=include_snapshot_ids,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ServiceResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_services_serialize(
        self,
        name_filter,
        window_size,
        to,
        page,
        page_size,
        include_snapshot_ids,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if name_filter is not None:
            
            _query_params.append(('nameFilter', name_filter))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('pageSize', page_size))
            
        if include_snapshot_ids is not None:
            
            _query_params.append(('includeSnapshotIds', include_snapshot_ids))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/application-monitoring/services',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import List, Optional
from typing_extensions import Annotated
from instana_client.models.get_snapshots_query import GetSnapshotsQuery
from instana_client.models.monitoring_state import MonitoringState
from instana_client.models.post_snapshots_result import PostSnapshotsResult
from instana_client.models.snapshot_item import SnapshotItem
from instana_client.models.snapshot_result import SnapshotResult
from instana_client.models.software_version import SoftwareVersion

from instana_client.api_client import ApiClient, RequestSerialized
from instana_client.api_response import ApiResponse
from instana_client.rest import RESTResponseType


class InfrastructureResourcesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def get_monitoring_state(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> MonitoringState:
        """Monitored host count

        This API endpoint retrieves the current monitoring state of the system, providing details about the number of monitored hosts and serverless entities. It responds with a JSON object containing this information.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_monitoring_state_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MonitoringState",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_monitoring_state_with_http_info(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[MonitoringState]:
        """Monitored host count

        This API endpoint retrieves the current monitoring state of the system, providing details about the number of monitored hosts and serverless entities. It responds with a JSON object containing this information.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_monitoring_state_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MonitoringState",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_monitoring_state_without_preload_content(
        self,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Monitored host count

        This API endpoint retrieves the current monitoring state of the system, providing details about the number of monitored hosts and serverless entities. It responds with a JSON object containing this information.

        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_monitoring_state_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MonitoringState",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_monitoring_state_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/infrastructure-monitoring/monitoring-state',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_plugin_payload(
        self,
        snapshot_id: Annotated[StrictStr, Field(description="Snapshot id.")],
        payload_key: Annotated[StrictStr, Field(description="Payload key. Use [getAvailablePayloadKeysByPluginId](/openapi/#operation/getAvailablePayloadKeysByPluginId) to retrieve the list of possible keys.")],
        to: Annotated[Optional[StrictInt], Field(description="End of timeframe expressed as the Unix epoch time in milliseconds.")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Window size in milliseconds.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Get a payload for a snapshot

        This endpoint retrieves the payload for a snapshot. Please note that this endpoint is only available for instances of db2.

        :param snapshot_id: Snapshot id. (required)
        :type snapshot_id: str
        :param payload_key: Payload key. Use [getAvailablePayloadKeysByPluginId](/openapi/#operation/getAvailablePayloadKeysByPluginId) to retrieve the list of possible keys. (required)
        :type payload_key: str
        :param to: End of timeframe expressed as the Unix epoch time in milliseconds.
        :type to: int
        :param window_size: Window size in milliseconds.
        :type window_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_plugin_payload_serialize(
            snapshot_id=snapshot_id,
            payload_key=payload_key,
            to=to,
            window_size=window_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_plugin_payload_with_http_info(
        self,
        snapshot_id: Annotated[StrictStr, Field(description="Snapshot id.")],
        payload_key: Annotated[StrictStr, Field(description="Payload key. Use [getAvailablePayloadKeysByPluginId](/openapi/#operation/getAvailablePayloadKeysByPluginId) to retrieve the list of possible keys.")],
        to: Annotated[Optional[StrictInt], Field(description="End of timeframe expressed as the Unix epoch time in milliseconds.")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Window size in milliseconds.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Get a payload for a snapshot

        This endpoint retrieves the payload for a snapshot. Please note that this endpoint is only available for instances of db2.

        :param snapshot_id: Snapshot id. (required)
        :type snapshot_id: str
        :param payload_key: Payload key. Use [getAvailablePayloadKeysByPluginId](/openapi/#operation/getAvailablePayloadKeysByPluginId) to retrieve the list of possible keys. (required)
        :type payload_key: str
        :param to: End of timeframe expressed as the Unix epoch time in milliseconds.
        :type to: int
        :param window_size: Window size in milliseconds.
        :type window_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_plugin_payload_serialize(
            snapshot_id=snapshot_id,
            payload_key=payload_key,
            to=to,
            window_size=window_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_plugin_payload_without_preload_content(
        self,
        snapshot_id: Annotated[StrictStr, Field(description="Snapshot id.")],
        payload_key: Annotated[StrictStr, Field(description="Payload key. Use [getAvailablePayloadKeysByPluginId](/openapi/#operation/getAvailablePayloadKeysByPluginId) to retrieve the list of possible keys.")],
        to: Annotated[Optional[StrictInt], Field(description="End of timeframe expressed as the Unix epoch time in milliseconds.")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="Window size in milliseconds.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get a payload for a snapshot

        This endpoint retrieves the payload for a snapshot. Please note that this endpoint is only available for instances of db2.

        :param snapshot_id: Snapshot id. (required)
        :type snapshot_id: str
        :param payload_key: Payload key. Use [getAvailablePayloadKeysByPluginId](/openapi/#operation/getAvailablePayloadKeysByPluginId) to retrieve the list of possible keys. (required)
        :type payload_key: str
        :param to: End of timeframe expressed as the Unix epoch time in milliseconds.
        :type to: int
        :param window_size: Window size in milliseconds.
        :type window_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_plugin_payload_serialize(
            snapshot_id=snapshot_id,
            payload_key=payload_key,
            to=to,
            window_size=window_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_plugin_payload_serialize(
        self,
        snapshot_id,
        payload_key,
        to,
        window_size,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if snapshot_id is not None:
            _path_params['snapshotId'] = snapshot_id
        if payload_key is not None:
            _path_params['payloadKey'] = payload_key
        # process the query parameters
        if to is not None:
            
            _query_params.append(('to', to))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/infrastructure-monitoring/payloads/{snapshotId}/{payloadKey}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_snapshot(
        self,
        id: Annotated[StrictStr, Field(description="snapshot id")],
        to: Annotated[Optional[StrictInt], Field(description="end of timeframe expressed as the Unix epoch time in milliseconds")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="windowSize in milliseconds")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SnapshotItem:
        """Get snapshot details

        Get all snapshot information

        :param id: snapshot id (required)
        :type id: str
        :param to: end of timeframe expressed as the Unix epoch time in milliseconds
        :type to: int
        :param window_size: windowSize in milliseconds
        :type window_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_snapshot_serialize(
            id=id,
            to=to,
            window_size=window_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SnapshotItem",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_snapshot_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="snapshot id")],
        to: Annotated[Optional[StrictInt], Field(description="end of timeframe expressed as the Unix epoch time in milliseconds")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="windowSize in milliseconds")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SnapshotItem]:
        """Get snapshot details

        Get all snapshot information

        :param id: snapshot id (required)
        :type id: str
        :param to: end of timeframe expressed as the Unix epoch time in milliseconds
        :type to: int
        :param window_size: windowSize in milliseconds
        :type window_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_snapshot_serialize(
            id=id,
            to=to,
            window_size=window_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SnapshotItem",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_snapshot_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="snapshot id")],
        to: Annotated[Optional[StrictInt], Field(description="end of timeframe expressed as the Unix epoch time in milliseconds")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="windowSize in milliseconds")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get snapshot details

        Get all snapshot information

        :param id: snapshot id (required)
        :type id: str
        :param to: end of timeframe expressed as the Unix epoch time in milliseconds
        :type to: int
        :param window_size: windowSize in milliseconds
        :type window_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_snapshot_serialize(
            id=id,
            to=to,
            window_size=window_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SnapshotItem",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_snapshot_serialize(
        self,
        id,
        to,
        window_size,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        if to is not None:
            
            _query_params.append(('to', to))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/infrastructure-monitoring/snapshots/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_snapshots(
        self,
        query: Annotated[Optional[StrictStr], Field(description="query to use to filter snapshot retrieval")] = None,
        to: Annotated[Optional[StrictInt], Field(description="end of timeframe expressed as the Unix epoch time in milliseconds")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="windowSize in milliseconds")] = None,
        size: Annotated[Optional[StrictInt], Field(description="maximum number of snapshots to retrieve")] = None,
        plugin: Annotated[Optional[StrictStr], Field(description="entity type")] = None,
        offline: Annotated[Optional[StrictBool], Field(description="retrieve snapshots which were online during the timeframe, otherwise, return only snapshot which were online at the end of the timeframe")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SnapshotResult:
        """Search snapshots


        :param query: query to use to filter snapshot retrieval
        :type query: str
        :param to: end of timeframe expressed as the Unix epoch time in milliseconds
        :type to: int
        :param window_size: windowSize in milliseconds
        :type window_size: int
        :param size: maximum number of snapshots to retrieve
        :type size: int
        :param plugin: entity type
        :type plugin: str
        :param offline: retrieve snapshots which were online during the timeframe, otherwise, return only snapshot which were online at the end of the timeframe
        :type offline: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_snapshots_serialize(
            query=query,
            to=to,
            window_size=window_size,
            size=size,
            plugin=plugin,
            offline=offline,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SnapshotResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_snapshots_with_http_info(
        self,
        query: Annotated[Optional[StrictStr], Field(description="query to use to filter snapshot retrieval")] = None,
        to: Annotated[Optional[StrictInt], Field(description="end of timeframe expressed as the Unix epoch time in milliseconds")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="windowSize in milliseconds")] = None,
        size: Annotated[Optional[StrictInt], Field(description="maximum number of snapshots to retrieve")] = None,
        plugin: Annotated[Optional[StrictStr], Field(description="entity type")] = None,
        offline: Annotated[Optional[StrictBool], Field(description="retrieve snapshots which were online during the timeframe, otherwise, return only snapshot which were online at the end of the timeframe")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SnapshotResult]:
        """Search snapshots


        :param query: query to use to filter snapshot retrieval
        :type query: str
        :param to: end of timeframe expressed as the Unix epoch time in milliseconds
        :type to: int
        :param window_size: windowSize in milliseconds
        :type window_size: int
        :param size: maximum number of snapshots to retrieve
        :type size: int
        :param plugin: entity type
        :type plugin: str
        :param offline: retrieve snapshots which were online during the timeframe, otherwise, return only snapshot which were online at the end of the timeframe
        :type offline: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_snapshots_serialize(
            query=query,
            to=to,
            window_size=window_size,
            size=size,
            plugin=plugin,
            offline=offline,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SnapshotResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_snapshots_without_preload_content(
        self,
        query: Annotated[Optional[StrictStr], Field(description="query to use to filter snapshot retrieval")] = None,
        to: Annotated[Optional[StrictInt], Field(description="end of timeframe expressed as the Unix epoch time in milliseconds")] = None,
        window_size: Annotated[Optional[StrictInt], Field(description="windowSize in milliseconds")] = None,
        size: Annotated[Optional[StrictInt], Field(description="maximum number of snapshots to retrieve")] = None,
        plugin: Annotated[Optional[StrictStr], Field(description="entity type")] = None,
        offline: Annotated[Optional[StrictBool], Field(description="retrieve snapshots which were online during the timeframe, otherwise, return only snapshot which were online at the end of the timeframe")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Search snapshots


        :param query: query to use to filter snapshot retrieval
        :type query: str
        :param to: end of timeframe expressed as the Unix epoch time in milliseconds
        :type to: int
        :param window_size: windowSize in milliseconds
        :type window_size: int
        :param size: maximum number of snapshots to retrieve
        :type size: int
        :param plugin: entity type
        :type plugin: str
        :param offline: retrieve snapshots which were online during the timeframe, otherwise, return only snapshot which were online at the end of the timeframe
        :type offline: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_snapshots_serialize(
            query=query,
            to=to,
            window_size=window_size,
            size=size,
            plugin=plugin,
            offline=offline,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SnapshotResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_snapshots_serialize(
        self,
        query,
        to,
        window_size,
        size,
        plugin,
        offline,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if query is not None:
            
            _query_params.append(('query', query))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if plugin is not None:
            
            _query_params.append(('plugin', plugin))
            
        if offline is not None:
            
            _query_params.append(('offline', offline))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/infrastructure-monitoring/snapshots',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def post_snapshots(
        self,
        get_snapshots_query: Optional[GetSnapshotsQuery] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PostSnapshotsResult:
        """Get snapshot details for multiple snapshots

        This endpoint retrieves detail information for one or more snapshots. timeFrame defaults to the last 10 minutes if not specified.

        :param get_snapshots_query:
        :type get_snapshots_query: GetSnapshotsQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_snapshots_serialize(
            get_snapshots_query=get_snapshots_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PostSnapshotsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def post_snapshots_with_http_info(
        self,
        get_snapshots_query: Optional[GetSnapshotsQuery] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PostSnapshotsResult]:
        """Get snapshot details for multiple snapshots

        This endpoint retrieves detail information for one or more snapshots. timeFrame defaults to the last 10 minutes if not specified.

        :param get_snapshots_query:
        :type get_snapshots_query: GetSnapshotsQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_snapshots_serialize(
            get_snapshots_query=get_snapshots_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PostSnapshotsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def post_snapshots_without_preload_content(
        self,
        get_snapshots_query: Optional[GetSnapshotsQuery] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get snapshot details for multiple snapshots

        This endpoint retrieves detail information for one or more snapshots. timeFrame defaults to the last 10 minutes if not specified.

        :param get_snapshots_query:
        :type get_snapshots_query: GetSnapshotsQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._post_snapshots_serialize(
            get_snapshots_query=get_snapshots_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PostSnapshotsResult",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _post_snapshots_serialize(
        self,
        get_snapshots_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if get_snapshots_query is not None:
            _body_params = get_snapshots_query


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/infrastructure-monitoring/snapshots',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def software_versions(
        self,
        time: Annotated[Optional[StrictInt], Field(description="Timeframe expressed as the Unix epoch time in milliseconds")] = None,
        discovery_type: Annotated[Optional[StrictStr], Field(description="Filter on discoveryType")] = None,
        software_type: Annotated[Optional[StrictStr], Field(description="Filter on softwareType")] = None,
        name: Annotated[Optional[StrictStr], Field(description="Filter on this software name")] = None,
        plugin: Annotated[Optional[StrictStr], Field(description="Filter on this plugin")] = None,
        version: Annotated[Optional[StrictStr], Field(description="Filter on this version")] = None,
        vendor: Annotated[Optional[StrictStr], Field(description="Filter on this vendor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[SoftwareVersion]:
        """Get installed software

        Retrieve information about the software that are sensed by the agent remotely, natively, or both. This includes runtime and package manager information.  The `plugin`, `name`, `version`, `discoveryType`, `softwareType` and `vendor` parameters are optional filters that can be used to reduce the result data set. The `snapshotId` in `usedBy` is either of host or container, if available

        :param time: Timeframe expressed as the Unix epoch time in milliseconds
        :type time: int
        :param discovery_type: Filter on discoveryType
        :type discovery_type: str
        :param software_type: Filter on softwareType
        :type software_type: str
        :param name: Filter on this software name
        :type name: str
        :param plugin: Filter on this plugin
        :type plugin: str
        :param version: Filter on this version
        :type version: str
        :param vendor: Filter on this vendor
        :type vendor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._software_versions_serialize(
            time=time,
            discovery_type=discovery_type,
            software_type=software_type,
            name=name,
            plugin=plugin,
            version=version,
            vendor=vendor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[SoftwareVersion]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def software_versions_with_http_info(
        self,
        time: Annotated[Optional[StrictInt], Field(description="Timeframe expressed as the Unix epoch time in milliseconds")] = None,
        discovery_type: Annotated[Optional[StrictStr], Field(description="Filter on discoveryType")] = None,
        software_type: Annotated[Optional[StrictStr], Field(description="Filter on softwareType")] = None,
        name: Annotated[Optional[StrictStr], Field(description="Filter on this software name")] = None,
        plugin: Annotated[Optional[StrictStr], Field(description="Filter on this plugin")] = None,
        version: Annotated[Optional[StrictStr], Field(description="Filter on this version")] = None,
        vendor: Annotated[Optional[StrictStr], Field(description="Filter on this vendor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[SoftwareVersion]]:
        """Get installed software

        Retrieve information about the software that are sensed by the agent remotely, natively, or both. This includes runtime and package manager information.  The `plugin`, `name`, `version`, `discoveryType`, `softwareType` and `vendor` parameters are optional filters that can be used to reduce the result data set. The `snapshotId` in `usedBy` is either of host or container, if available

        :param time: Timeframe expressed as the Unix epoch time in milliseconds
        :type time: int
        :param discovery_type: Filter on discoveryType
        :type discovery_type: str
        :param software_type: Filter on softwareType
        :type software_type: str
        :param name: Filter on this software name
        :type name: str
        :param plugin: Filter on this plugin
        :type plugin: str
        :param version: Filter on this version
        :type version: str
        :param vendor: Filter on this vendor
        :type vendor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._software_versions_serialize(
            time=time,
            discovery_type=discovery_type,
            software_type=software_type,
            name=name,
            plugin=plugin,
            version=version,
            vendor=vendor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[SoftwareVersion]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def software_versions_without_preload_content(
        self,
        time: Annotated[Optional[StrictInt], Field(description="Timeframe expressed as the Unix epoch time in milliseconds")] = None,
        discovery_type: Annotated[Optional[StrictStr], Field(description="Filter on discoveryType")] = None,
        software_type: Annotated[Optional[StrictStr], Field(description="Filter on softwareType")] = None,
        name: Annotated[Optional[StrictStr], Field(description="Filter on this software name")] = None,
        plugin: Annotated[Optional[StrictStr], Field(description="Filter on this plugin")] = None,
        version: Annotated[Optional[StrictStr], Field(description="Filter on this version")] = None,
        vendor: Annotated[Optional[StrictStr], Field(description="Filter on this vendor")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get installed software

        Retrieve information about the software that are sensed by the agent remotely, natively, or both. This includes runtime and package manager information.  The `plugin`, `name`, `version`, `discoveryType`, `softwareType` and `vendor` parameters are optional filters that can be used to reduce the result data set. The `snapshotId` in `usedBy` is either of host or container, if available

        :param time: Timeframe expressed as the Unix epoch time in milliseconds
        :type time: int
        :param discovery_type: Filter on discoveryType
        :type discovery_type: str
        :param software_type: Filter on softwareType
        :type software_type: str
        :param name: Filter on this software name
        :type name: str
        :param plugin: Filter on this plugin
        :type plugin: str
        :param version: Filter on this version
        :type version: str
        :param vendor: Filter on this vendor
        :type vendor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._software_versions_serialize(
            time=time,
            discovery_type=discovery_type,
            software_type=software_type,
            name=name,
            plugin=plugin,
            version=version,
            vendor=vendor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[SoftwareVersion]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _software_versions_serialize(
        self,
        time,
        discovery_type,
        software_type,
        name,
        plugin,
        version,
        vendor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if time is not None:
            
            _query_params.append(('time', time))
            
        if discovery_type is not None:
            
            _query_params.append(('discoveryType', discovery_type))
            
        if software_type is not None:
            
            _query_params.append(('softwareType', software_type))
            
        if name is not None:
            
            _query_params.append(('name', name))
            
        if plugin is not None:
            
            _query_params.append(('plugin', plugin))
            
        if version is not None:
            
            _query_params.append(('version', version))
            
        if vendor is not None:
            
            _query_params.append(('vendor', vendor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/infrastructure-monitoring/software/versions',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class CorrectionScheduling(BaseModel):
    """
    Time scheduling of the SLO Correction Window configurations.
    """ # noqa: E501
    duration: StrictInt = Field(description="Duration of the correction window, defined by the count of time units.")
    duration_unit: StrictStr = Field(description="Unit for duration. Supported values: `MINUTE`, `HOUR`, `DAY`.", alias="durationUnit")
    recurrent: Optional[StrictBool] = None
    recurrent_rule: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=2048)]] = Field(default=None, description="Recurrence Rule defines how the event recurs, using iCalendar RRULE format (e.g., `FREQ=DAILY;INTERVAL=1`).It supports fields like FREQ, INTERVAL, BYDAY, etc. Refer to RFC 5545: https://datatracker.ietf.org/doc/html/rfc5545#section-3.3.10", alias="recurrentRule")
    start_time: Optional[datetime] = Field(default=None, description="Start time of the correction window in milliseconds since epoch (UTC) (e.g., `1706713140000`).", alias="startTime")
    __properties: ClassVar[List[str]] = ["duration", "durationUnit", "recurrent", "recurrentRule", "startTime"]

    @field_validator('duration_unit')
    def duration_unit_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['millisecond', 'second', 'minute', 'hour', 'day', 'week', 'month']):
            raise ValueError("must be one of enum values ('millisecond', 'second', 'minute', 'hour', 'day', 'week', 'month')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CorrectionScheduling from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CorrectionScheduling from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "duration": obj.get("duration"),
            "durationUnit": obj.get("durationUnit"),
            "recurrent": obj.get("recurrent"),
            "recurrentRule": obj.get("recurrentRule"),
            "startTime": obj.get("startTime")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.action_configuration import ActionConfiguration

class TestActionConfiguration(unittest.TestCase):
    """ActionConfiguration unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ActionConfiguration:
        """Test ActionConfiguration
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ActionConfiguration`
        """
        model = ActionConfiguration()
        if include_optional:
            return ActionConfiguration(
                action = instana_client.models.action.Action(
                    created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    description = '', 
                    fields = [
                        instana_client.models.field.Field(
                            description = '', 
                            encoding = '', 
                            name = '', 
                            secured = True, 
                            value = '', )
                        ], 
                    id = '0', 
                    input_parameters = [
                        instana_client.models.parameter.Parameter(
                            description = '', 
                            hidden = True, 
                            label = '', 
                            name = '', 
                            required = True, 
                            secured = True, 
                            type = '', 
                            value = '', )
                        ], 
                    metadata = instana_client.models.meta_data.MetaData(
                        ai = [
                            {
                                'key' : None
                                }
                            ], 
                        ai_originated = True, 
                        built_in = True, 
                        read_only = True, 
                        sensor_imported = True, ), 
                    modified_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    name = '0', 
                    tags = [
                        ''
                        ], 
                    type = 'SCRIPT', ),
                agent_id = '',
                input_parameter_values = [
                    instana_client.models.parameter_value.ParameterValue(
                        name = '', 
                        value = '', )
                    ]
            )
        else:
            return ActionConfiguration(
                action = instana_client.models.action.Action(
                    created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    description = '', 
                    fields = [
                        instana_client.models.field.Field(
                            description = '', 
                            encoding = '', 
                            name = '', 
                            secured = True, 
                            value = '', )
                        ], 
                    id = '0', 
                    input_parameters = [
                        instana_client.models.parameter.Parameter(
                            description = '', 
                            hidden = True, 
                            label = '', 
                            name = '', 
                            required = True, 
                            secured = True, 
                            type = '', 
                            value = '', )
                        ], 
                    metadata = instana_client.models.meta_data.MetaData(
                        ai = [
                            {
                                'key' : None
                                }
                            ], 
                        ai_originated = True, 
                        built_in = True, 
                        read_only = True, 
                        sensor_imported = True, ), 
                    modified_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    name = '0', 
                    tags = [
                        ''
                        ], 
                    type = 'SCRIPT', ),
        )
        """

    def testActionConfiguration(self):
        """Test ActionConfiguration"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from importlib import import_module
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from instana_client.models.browser_script_configuration_update import BrowserScriptConfigurationUpdate
    from instana_client.models.dns_configuration_update import DNSConfigurationUpdate
    from instana_client.models.http_action_configuration_update import HttpActionConfigurationUpdate
    from instana_client.models.http_script_configuration_update import HttpScriptConfigurationUpdate
    from instana_client.models.ssl_certificate_configuration_update import SSLCertificateConfigurationUpdate
    from instana_client.models.webpage_action_configuration_update import WebpageActionConfigurationUpdate
    from instana_client.models.webpage_script_configuration_update import WebpageScriptConfigurationUpdate

class SyntheticTypeConfigurationUpdate(BaseModel):
    """
    Synthetic test configuration that is unique to a synthetic type. Valid types are BrowserScript, DNS, HTTPAction, HTTPScript, SSLCertificate, WebpageAction, and WebpageScript.
    """ # noqa: E501
    mark_synthetic_call: Optional[StrictBool] = Field(default=None, alias="markSyntheticCall")
    retries: Optional[Annotated[int, Field(le=2, strict=True, ge=0)]] = None
    retry_interval: Optional[Annotated[int, Field(le=10, strict=True, ge=1)]] = Field(default=None, alias="retryInterval")
    synthetic_test: Optional[StrictStr] = Field(default=None, alias="syntheticTest")
    synthetic_type: Optional[StrictStr] = Field(default=None, alias="syntheticType")
    timeout: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["markSyntheticCall", "retries", "retryInterval", "syntheticTest", "syntheticType", "timeout"]

    @field_validator('synthetic_test')
    def synthetic_test_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['BrowserScript', 'DNS', 'HTTPAction', 'HTTPScript', 'SSLCertificate', 'WebpageAction', 'WebpageScript', 'NotConfigured']):
            raise ValueError("must be one of enum values ('BrowserScript', 'DNS', 'HTTPAction', 'HTTPScript', 'SSLCertificate', 'WebpageAction', 'WebpageScript', 'NotConfigured')")
        return value

    @field_validator('synthetic_type')
    def synthetic_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['BrowserScript', 'DNS', 'HTTPAction', 'HTTPScript', 'SSLCertificate', 'WebpageAction', 'WebpageScript', 'NotConfigured']):
            raise ValueError("must be one of enum values ('BrowserScript', 'DNS', 'HTTPAction', 'HTTPScript', 'SSLCertificate', 'WebpageAction', 'WebpageScript', 'NotConfigured')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = 'syntheticType'

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        'BrowserScript': 'BrowserScriptConfigurationUpdate','DNS': 'DNSConfigurationUpdate','HTTPAction': 'HttpActionConfigurationUpdate','HTTPScript': 'HttpScriptConfigurationUpdate','SSLCertificate': 'SSLCertificateConfigurationUpdate','WebpageAction': 'WebpageActionConfigurationUpdate','WebpageScript': 'WebpageScriptConfigurationUpdate'
    }

    @classmethod
    def get_discriminator_value(cls, obj: Dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Union[BrowserScriptConfigurationUpdate, DNSConfigurationUpdate, HttpActionConfigurationUpdate, HttpScriptConfigurationUpdate, SSLCertificateConfigurationUpdate, WebpageActionConfigurationUpdate, WebpageScriptConfigurationUpdate]]:
        """Create an instance of SyntheticTypeConfigurationUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Optional[Union[BrowserScriptConfigurationUpdate, DNSConfigurationUpdate, HttpActionConfigurationUpdate, HttpScriptConfigurationUpdate, SSLCertificateConfigurationUpdate, WebpageActionConfigurationUpdate, WebpageScriptConfigurationUpdate]]:
        """Create an instance of SyntheticTypeConfigurationUpdate from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type ==  'BrowserScriptConfigurationUpdate':
            return import_module("instana_client.models.browser_script_configuration_update").BrowserScriptConfigurationUpdate.from_dict(obj)
        if object_type ==  'DNSConfigurationUpdate':
            return import_module("instana_client.models.dns_configuration_update").DNSConfigurationUpdate.from_dict(obj)
        if object_type ==  'HttpActionConfigurationUpdate':
            return import_module("instana_client.models.http_action_configuration_update").HttpActionConfigurationUpdate.from_dict(obj)
        if object_type ==  'HttpScriptConfigurationUpdate':
            return import_module("instana_client.models.http_script_configuration_update").HttpScriptConfigurationUpdate.from_dict(obj)
        if object_type ==  'SSLCertificateConfigurationUpdate':
            return import_module("instana_client.models.ssl_certificate_configuration_update").SSLCertificateConfigurationUpdate.from_dict(obj)
        if object_type ==  'WebpageActionConfigurationUpdate':
            return import_module("instana_client.models.webpage_action_configuration_update").WebpageActionConfigurationUpdate.from_dict(obj)
        if object_type ==  'WebpageScriptConfigurationUpdate':
            return import_module("instana_client.models.webpage_script_configuration_update").WebpageScriptConfigurationUpdate.from_dict(obj)

        raise ValueError("SyntheticTypeConfigurationUpdate failed to lookup discriminator value from " +
                            json.dumps(obj) + ". Discriminator property name: " + cls.__discriminator_property_name +
                            ", mapping: " + json.dumps(cls.__discriminator_value_class_map))





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.mobile_app_beacon_result import MobileAppBeaconResult

class TestMobileAppBeaconResult(unittest.TestCase):
    """MobileAppBeaconResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> MobileAppBeaconResult:
        """Test MobileAppBeaconResult
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `MobileAppBeaconResult`
        """
        model = MobileAppBeaconResult()
        if include_optional:
            return MobileAppBeaconResult(
                adjusted_timeframe = instana_client.models.adjusted_timeframe.AdjustedTimeframe(
                    to = 56, 
                    window_size = 0, ),
                can_load_more = True,
                items = [
                    instana_client.models.mobile_app_beacons_item.MobileAppBeaconsItem(
                        beacon = instana_client.models.mobile_app_monitoring_beacon.MobileAppMonitoringBeacon(
                            accuracy_radius = -1, 
                            agent_version = '', 
                            app_build = '', 
                            app_version = '', 
                            app_version_number = -1, 
                            available_mb = -1, 
                            backend_trace_id = '', 
                            batch_size = 1, 
                            beacon_id = '', 
                            bundle_identifier = '', 
                            bytes_ingested = 56, 
                            carrier = '', 
                            city = '', 
                            clock_skew = 0, 
                            cold_start_time_ms = -1, 
                            connection_type = '', 
                            continent = '', 
                            continent_code = '', 
                            country = '', 
                            country_code = '', 
                            current_app_state = '', 
                            custom_event_name = '', 
                            custom_metric = 1.337, 
                            decoded_body_size = -1, 
                            device_hardware = '', 
                            device_manufacturer = '', 
                            device_model = '', 
                            drop_view = '', 
                            duration = 0, 
                            effective_connection_type = '', 
                            encoded_body_size = -1, 
                            environment = '', 
                            error_count = 0, 
                            error_id = '', 
                            error_message = '', 
                            error_type = '', 
                            google_play_services_missing = True, 
                            hot_start_time_ms = -1, 
                            http_call_headers = {
                                'key' : ''
                                }, 
                            http_call_method = '', 
                            http_call_origin = '', 
                            http_call_path = '', 
                            http_call_status = -1, 
                            http_call_url = '', 
                            ingestion_time = 1, 
                            internal_meta = {
                                'key' : ''
                                }, 
                            label = '', 
                            latitude = 1.337, 
                            longitude = 1.337, 
                            max_mb = -1, 
                            meta = {
                                'key' : ''
                                }, 
                            mobile_app_id = '', 
                            mobile_app_label = '', 
                            os_name = '', 
                            os_version = '', 
                            parent_beacon_id = '', 
                            parsed_stack_trace = '', 
                            performance_subtype = '', 
                            platform = '', 
                            rate_limit_beacon_type = '', 
                            rate_limit_count = 0, 
                            rate_limit_custom_metric_val = [
                                1.337
                                ], 
                            rate_limit_time_max = 0, 
                            rate_limit_time_min = 0, 
                            region = '', 
                            rooted = True, 
                            session_id = '', 
                            stack_trace = '', 
                            stack_trace_key_checksum = '', 
                            stack_trace_key_information = '', 
                            stack_trace_line = [
                                instana_client.models.stack_trace_line.StackTraceLine(
                                    column = -1, 
                                    file = '', 
                                    line = -1, 
                                    name = '', 
                                    translation_explanation = '', 
                                    translation_status = -1, )
                                ], 
                            stack_trace_parsing_status = -1, 
                            subdivision = '', 
                            subdivision_code = '', 
                            tenant = '', 
                            timestamp = 1, 
                            transfer_size = -1, 
                            type = '', 
                            unit = '', 
                            use_features = [
                                ''
                                ], 
                            used_mb = -1, 
                            user_email = '', 
                            user_id = '', 
                            user_ip = '', 
                            user_languages = [
                                ''
                                ], 
                            user_name = '', 
                            user_session_id = '', 
                            view = '', 
                            viewport_height = -1, 
                            viewport_width = -1, 
                            warm_start_time_ms = -1, ), 
                        cursor = instana_client.models.cursor.cursor(), 
                        impacted_beacon_info = instana_client.models.impacted_beacon_info.ImpactedBeaconInfo(
                            country = '', 
                            label = '', 
                            subdivision = '', 
                            user_email = '', 
                            user_name = '', ), )
                    ],
                total_hits = 0,
                total_represented_item_count = 0,
                total_retained_item_count = 0
            )
        else:
            return MobileAppBeaconResult(
                items = [
                    instana_client.models.mobile_app_beacons_item.MobileAppBeaconsItem(
                        beacon = instana_client.models.mobile_app_monitoring_beacon.MobileAppMonitoringBeacon(
                            accuracy_radius = -1, 
                            agent_version = '', 
                            app_build = '', 
                            app_version = '', 
                            app_version_number = -1, 
                            available_mb = -1, 
                            backend_trace_id = '', 
                            batch_size = 1, 
                            beacon_id = '', 
                            bundle_identifier = '', 
                            bytes_ingested = 56, 
                            carrier = '', 
                            city = '', 
                            clock_skew = 0, 
                            cold_start_time_ms = -1, 
                            connection_type = '', 
                            continent = '', 
                            continent_code = '', 
                            country = '', 
                            country_code = '', 
                            current_app_state = '', 
                            custom_event_name = '', 
                            custom_metric = 1.337, 
                            decoded_body_size = -1, 
                            device_hardware = '', 
                            device_manufacturer = '', 
                            device_model = '', 
                            drop_view = '', 
                            duration = 0, 
                            effective_connection_type = '', 
                            encoded_body_size = -1, 
                            environment = '', 
                            error_count = 0, 
                            error_id = '', 
                            error_message = '', 
                            error_type = '', 
                            google_play_services_missing = True, 
                            hot_start_time_ms = -1, 
                            http_call_headers = {
                                'key' : ''
                                }, 
                            http_call_method = '', 
                            http_call_origin = '', 
                            http_call_path = '', 
                            http_call_status = -1, 
                            http_call_url = '', 
                            ingestion_time = 1, 
                            internal_meta = {
                                'key' : ''
                                }, 
                            label = '', 
                            latitude = 1.337, 
                            longitude = 1.337, 
                            max_mb = -1, 
                            meta = {
                                'key' : ''
                                }, 
                            mobile_app_id = '', 
                            mobile_app_label = '', 
                            os_name = '', 
                            os_version = '', 
                            parent_beacon_id = '', 
                            parsed_stack_trace = '', 
                            performance_subtype = '', 
                            platform = '', 
                            rate_limit_beacon_type = '', 
                            rate_limit_count = 0, 
                            rate_limit_custom_metric_val = [
                                1.337
                                ], 
                            rate_limit_time_max = 0, 
                            rate_limit_time_min = 0, 
                            region = '', 
                            rooted = True, 
                            session_id = '', 
                            stack_trace = '', 
                            stack_trace_key_checksum = '', 
                            stack_trace_key_information = '', 
                            stack_trace_line = [
                                instana_client.models.stack_trace_line.StackTraceLine(
                                    column = -1, 
                                    file = '', 
                                    line = -1, 
                                    name = '', 
                                    translation_explanation = '', 
                                    translation_status = -1, )
                                ], 
                            stack_trace_parsing_status = -1, 
                            subdivision = '', 
                            subdivision_code = '', 
                            tenant = '', 
                            timestamp = 1, 
                            transfer_size = -1, 
                            type = '', 
                            unit = '', 
                            use_features = [
                                ''
                                ], 
                            used_mb = -1, 
                            user_email = '', 
                            user_id = '', 
                            user_ip = '', 
                            user_languages = [
                                ''
                                ], 
                            user_name = '', 
                            user_session_id = '', 
                            view = '', 
                            viewport_height = -1, 
                            viewport_width = -1, 
                            warm_start_time_ms = -1, ), 
                        cursor = instana_client.models.cursor.cursor(), 
                        impacted_beacon_info = instana_client.models.impacted_beacon_info.ImpactedBeaconInfo(
                            country = '', 
                            label = '', 
                            subdivision = '', 
                            user_email = '', 
                            user_name = '', ), )
                    ],
        )
        """

    def testMobileAppBeaconResult(self):
        """Test MobileAppBeaconResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
